@@ -13,7 +13,9 @@ async def restore_main(args):
         transaction=args.transaction,
     )
 
-    async with asyncpg.create_pool() as pool:
+    async with asyncpg.create_pool(
+        min_size=0, max_size=args.jobs, max_inactive_connection_lifetime=10
+    ) as pool:
         io = RestoreIo(
             conn=lambda: pool.acquire(), input=lambda: open_bytes_read(args.input)
         )
